@@ -141,9 +141,9 @@ class auto_document:
                 source_code,arguments,this_returns,description = self.method_or_function_insight(moduleOrLibraryInstance,a,b)
                 source_code  = source_code.replace('"','\\"')
                 rslt[a][b] = {}
-                rslt[a][b]["source_code"] = source_code
+                #rslt[a][b]["source_code"] = source_code
                 rslt[a][b]["arguments"] = arguments
-                rslt[a][b]["returns"] = this_returns
+                rslt[a][b]["returns"] = this_returns.replace('Union[','').replace(']','')
                 rslt[a][b]["description"] = description.replace("acb123*acb123*acb123*","***")
                 rslt[a][b]["dependencies"] = self.source_code_dependencies(
                     moduleOrLibraryInstance=moduleOrLibraryInstance
@@ -160,10 +160,6 @@ class auto_document:
             self.tools.standard_dict_to_json(jsonOrDictionary=rslt
                                              ,fileName=filename_without_ext + ".json"
                                              ,folderPath=self.tools.documentation_JSON_path)
-            if not os.path.exists(self.tools.documentation_JS_path):
-                os.makedirs(self.tools.documentation_JS_path)
-            with open(self.tools.documentation_JS_path + filename_without_ext +  '.js','w') as f:
-                f.write("const " + filename_without_ext + " = " + str(rslt) + ";")
             self.load_classes_and_methods()
             print("Documentation sucessfully created at:\n" + self.tools.documentation_path)
         except Exception as e:
@@ -242,8 +238,7 @@ class processing_tools:
             self.sta_bucket =  's3://' + bucket_name + '/'
             self.environment = bucket_name
         self.documentation_path = self.local_directory + 'documentation/'
-        self.documentation_JSON_path = self.documentation_path + "json/" 
-        self.documentation_JS_path = self.documentation_path + "js/" 
+        self.documentation_JSON_path = self.documentation_path + "json/"  
         self.settings = self.sta_bucket + 'settings/'
         self.secrets = self.settings + 'secrets/'
         self.metadata = self.settings + 'metadata/'
@@ -408,7 +403,7 @@ class processing_tools:
             files = [f for f in files if f.endswith('.' + extension)]
         return set(files)
     #__________________________________________________________________________ 
-    def store_str_as_file_in_s3_folder(self,strInput,fileName,s3FullFolderPath)->None:
+    def store_str_as_file_in_s3_folder(self,strInput:str,fileName:str,s3FullFolderPath:str)->None:
         '''
         ***
         Stores a given strInput with a given fileName in a given s3FullFolderPath.
@@ -440,7 +435,7 @@ class processing_tools:
             rslt = "0" + rslt
         return rslt
     #__________________________________________________________________________
-    def standard_dict_to_json(self,jsonOrDictionary,fileName,folderPath)->None:
+    def standard_dict_to_json(self,jsonOrDictionary:dict,fileName:str,folderPath:str)->None:
         '''
         Stores a given dic as a formatted json file in either an s3 or a local depending on the instance variable self.environment value which can be 'local'
         or 's3'. 
@@ -489,7 +484,7 @@ class processing_tools:
         timestr = hh + ':' + mm + ':' + ss
         return timestr
     #__________________________________________________________________________
-    def get_s3_file_content(self,referenceName,s3FullFolderPath,exceptionCase=False)->str:
+    def get_s3_file_content(self,referenceName:str,s3FullFolderPath:str,exceptionCase=False)->str:
         '''
         ***
         Returns the content of given s3 file reference name within a given s3 folder location.
@@ -527,7 +522,7 @@ class processing_tools:
             else:
                 return fileContent
     #__________________________________________________________________________
-    def find_files_in_s3_folder(self,s3FullFolderPath)->set:
+    def find_files_in_s3_folder(self,s3FullFolderPath:str)->set:
         '''
         ***
         Return the set of folders and files found inside a given s3 location.
@@ -552,7 +547,7 @@ class processing_tools:
             these_files.add(thisFile)
         return these_files             
     #__________________________________________________________________________ 
-    def store_api_staging_data(self,symbol)->Union[str,str]:
+    def store_api_staging_data(self,symbol:str)->Union[str,str]:
         '''
         ***
         Description not available.
@@ -679,7 +674,7 @@ class processing_tools:
         else:
             return "encryption expired"
     #__________________________________________________________________________
-    def pre_gen_encrypted_data_with_expiration(self,inputStr)->dict:
+    def pre_gen_encrypted_data_with_expiration(self,inputStr:str)->dict:
         '''
         ***
         Description not available.
@@ -742,7 +737,7 @@ class processing_tools:
         else:
             return False
     #__________________________________________________________________________
-    def alpha_ofuscate(self,intOrStrInput)->str:
+    def alpha_ofuscate(self,intOrStrInput:str)->str:
         '''
         ***
         Description not available.
@@ -775,7 +770,7 @@ class processing_tools:
         rslt = a+keystr[::-1] + b + "<****>" + encrypted_message
         return rslt
     #__________________________________________________________________________
-    def extract_seconds_from_encrypted_input(self,strInput)->Union[int,str]:
+    def extract_seconds_from_encrypted_input(self,strInput:str)->Union[int,str]:
         '''
         ***
         Description not available.
@@ -822,4 +817,62 @@ class processing_tools:
             except:
                 ...
         return data
+    #__________________________________________________________________________
+    def popsql(self,df_input_name:str,df_output_name:str,column_names:object,action_expression:str,new_column_names:object=None)->pd.DataFrame:
+        '''
+        ***
+        Allows to apply a python expression to a column_name argument. Therefore, the dynamic python code should be provided as string with action over the 
+        keyword "@arg". 
+        The resultant of the python action can be in the same column(s) or in new ones, defined with the new_columns_name parameter.
+        column_names and new_column_names can be str or list depending if the change will apply to a single or multiple columns respectively.
+        It simplifies the creation of a user defined function on pandas dataframes.
+        Example:
+        supuse df is a pandas dataframe and one of it´s column names is "Column A".
+         +----+----------------+
+         | id | Column A       |
+         +----+----------------+
+         |  1 |  Something     |
+         |  2 |  Something Else|
+         |  3 |  xyZ           |
+         +----+----------------+
+        If we aply python action with givern action_expression:
+        df_python_action(
+            dataframe = df
+            ,column_names = "Column A"
+            ,action_expression = "@arg.lower() + ' hello'"
+            ,new_column_names = "New Column A"
+            )
+        The result would look like:
+         +----+----------------+----------------------+
+         | id | Column A       | New Column A         |
+         +----+----------------+----------------------+
+         |  1 |  Something     |  something hello     |
+         |  2 |  Something Else|  something else hello|
+         |  3 |  xyZ           |  xyz hello           |
+         +----+----------------+----------------------+
+        ***
+        '''
+        
+        if type(column_names) == str:
+            column_names_obj = '"' + column_names + '"'
+        else:
+            column_names_obj = str(column_names)
+        if type(new_column_names) == str:
+            new_column_names_obj = '"' + new_column_names + '"'
+        else:
+            new_column_names_obj = str(new_column_names)   
+        dynamyc_code = '@df_output_name = ' + df_input_name + '.copy()\n'\
+        'column_names,new_column_names= ' + column_names_obj + ',' + new_column_names_obj + '\n'\
+        'if new_column_names == None:\n'\
+        '\tnew_column_names = column_names\n'\
+        'if type(column_names) == list and type(new_column_names) == list:\n'\
+        '\tif len(new_column_names) == len(column_names):\n'\
+        '\t\tfor i in range(len(new_column_names)):\n'\
+        '\t\t\t@df_output_name[new_column_names[i]] = @df_output_name[column_names[i]].apply(lambda x: pd.Series(@action_expression))\n'\
+        'elif type(column_names) == str and type(new_column_names) == str:\n'\
+        '\t@df_output_name[new_column_names] = @df_output_name[column_names].apply(lambda x: pd.Series(@action_expression))\n'\
+        'else:\n'\
+        '\tprint(\'Arguments "column_names" and "new_column_names" can only be str or list and must be of the same type between each other.\')\n'
+        dynamyc_code = dynamyc_code.replace('@action_expression',action_expression).replace('@arg','x').replace("@df_output_name",df_output_name)
+        return(dynamyc_code)
     #__________________________________________________________________________
\ No newline at end of file
